//=======================================================================
//			Copyright (C) Shambler Team 2004
//		         util_.cpp - utility entities: util_fade, 
//			       util_shake, util_fov etc			    
//=======================================================================

#include "extdll.h"
#include "utils.h"
#include "cbase.h"
#include "saverestore.h"
#include "player.h"
#include "defaults.h"
#include "shake.h"

//=======================================================================
// util_setcamera - set view at entity for player
//=======================================================================
class CSetCamera : public CBaseLogic
{
	void Spawn (void );
	void UpdatePlayerView( void );
	void Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value );
	void KeyValue( KeyValueData* );
	int ObjectCaps( void ) { return CBaseLogic::ObjectCaps() & ~FCAP_ACROSS_TRANSITION; }
	char* GetStringForMode( void );
	
};
LINK_ENTITY_TO_CLASS( util_setcamera,  CSetCamera );

void CSetCamera :: KeyValue( KeyValueData* pkvd )
{
	if (FStrEq(pkvd->szKeyName, "lookat"))
	{
		pev->netname = ALLOC_STRING(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else if (FStrEq(pkvd->szKeyName, "hudmode"))
	{
		pev->skin = atoi(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else	CBaseEntity::KeyValue( pkvd );
}

void CSetCamera :: Spawn (void )
{
	pev->solid	         	= SOLID_NOT;
	pev->movetype		= MOVETYPE_NONE;

	switch (pev->skin)
	{
	case 1:	pev->button = 35;	break;//draw normal hud
	case 2:	pev->button = 1;	break;//warhead hud
	case 3:	pev->button = 2;	break;//warhead underwater hud
	case 4:	pev->button = 3;	break;//warhead noise screen
	default:	pev->button = 0;	break;//no hud
	}
	m_iState = STATE_OFF;
}

void CSetCamera :: UpdatePlayerView( void )
{
	int flags;

	((CBasePlayer *)((CBaseEntity *)m_hActivator))->EnableControl(GetState() ? FALSE : TRUE );
	((CBasePlayer *)((CBaseEntity *)m_hActivator))->m_iWarHUD = pev->button;
	if(GetState() == STATE_OFF) flags = 0;
	else flags = (pev->button == 35 ? DRAW_HUD | CAMERA_ON : CAMERA_ON); 
	UTIL_SetView( (CBaseEntity *)m_hActivator, pev->netname, flags );

}

void CSetCamera :: Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value )
{
	if(pActivator && pActivator->IsPlayer())//only at player
	{
		m_hActivator = pActivator;//save activator
		
		if (useType == USE_TOGGLE)
		{
			if(m_iState == STATE_OFF) useType = USE_ON;
			else useType = USE_OFF;
		}
		if (useType == USE_ON )
		{
			m_iState = STATE_ON;
			UpdatePlayerView();
		}
		else if (useType == USE_OFF)
		{
			m_iState = STATE_OFF;
			UpdatePlayerView();
		}

	}
	else if (useType == USE_SET)
	{
		//Find func_monitor
		CBaseEntity *pMonitor = UTIL_FindEntityByTargetname(NULL, STRING(pev->target));
		if(pMonitor)
		{
			pMonitor->pev->netname = pev->netname;
			pMonitor->PostActivate();
		}
	}
	else if (useType == USE_RESET)//this is' little easter egg - toggle screen view from player
	{
		//Find func_monitor
		CBaseEntity *pMonitor = UTIL_FindEntityByTargetname(NULL, STRING(pev->target));
		if(pMonitor)
		{
			pMonitor->pev->netname = MAKE_STRING( "player" );
			pMonitor->PostActivate();
		}
	}
	else if (useType == USE_SHOWINFO)
	{
		ALERT(at_console, "======/Xash Debug System/======\n");
		ALERT(at_console, "classname: %s\n", STRING(pev->classname));
		ALERT(at_console, "State: %s, Look at %s\n", GetStringForState( GetState()), STRING(pev->target));
		ALERT(at_console, "HUD Mode: %s\n", GetStringForMode);
	}
}

char* CSetCamera :: GetStringForMode( void )
{
	switch(pev->button)
	{
		case 0: return "Disable HUD";
		case 1: return "Draw Redeemer HUD";
		case 2: return "Draw Redeemer Underwater HUD";
		case 3: return "Draw Redeemer Noise Screen";
		case 35: return "Draw HUD";
	default:	return "";
	}
}

//=======================================================================
// func_mirror - breakable mirror brush 
//=======================================================================
class CFuncMirror : public CBaseBrush
{
public:
	void Spawn( void );
	void PostActivate( void );
	void Precache( void ){ CBaseBrush::Precache(); }
	void KeyValue( KeyValueData *pkvd );
	void Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value );
	virtual int ObjectCaps( void ) { return CBaseEntity :: ObjectCaps() & ~FCAP_ACROSS_TRANSITION; }
	virtual STATE GetState( void ) { return pev->body ? STATE_ON:STATE_OFF; };
};
LINK_ENTITY_TO_CLASS( func_mirror, CFuncMirror );

void CFuncMirror::KeyValue( KeyValueData *pkvd )
{
	if (FStrEq(pkvd->szKeyName, "alpha"))
	{
		pev->scale = atof( pkvd->szValue );
		pkvd->fHandled = TRUE;
	}
	else	CBaseBrush::KeyValue( pkvd );
}

void CFuncMirror :: Spawn( void )
{
	m_Material = Glass;
	CBaseBrush::Spawn();
	if (!m_pParent) pev->flags = FL_WORLDBRUSH;

	if(IsMultiplayer())
	{
		ALERT(at_console, "\n======/Xash SmartFiled System/======\n\n");
		ALERT(at_console, "Sorry, %s don't working in multiplayer\nPlease, use static world mirrors\n", STRING(pev->classname));
		SHIFT;
	}
	UTIL_SetModel( ENT(pev), pev->model ); 
	if(pev->spawnflags & SF_NOTSOLID)//make illusionary wall 
	{
		pev->solid = SOLID_NOT;
		pev->movetype = MOVETYPE_NONE;
	}
	else
	{
		pev->solid = SOLID_BSP;
          	pev->movetype = MOVETYPE_PUSH;
	}
	pev->body = 1;
}

void CFuncMirror :: PostActivate( void )
{
	MESSAGE_BEGIN( MSG_ALL, gmsgAddMirror );
		WRITE_BYTE( entindex() );
	MESSAGE_END();
}

void CFuncMirror :: Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value )
{
	if (useType == USE_TOGGLE)
	{
		if(pev->body) useType = USE_OFF;
		else useType = USE_ON;
	}
	if (useType == USE_ON) pev->body = TRUE;
	if (useType == USE_OFF)pev->body = FALSE;
	if (useType == USE_SET)Die();
	if (useType == USE_SHOWINFO)
	{
		DEBUGHEAD;
		ALERT(at_console, "State: %s, alpha %.f\n", GetStringForState( GetState()), pev->scale);
		ALERT(at_console, "Renderamt: %.f, Health %.f\n", pev->renderamt, pev->health);
	}
}

//=======================================================================
// func_monitor - A monitor that renders the view from a camera entity. 
//=======================================================================
class CFuncMonitor : public CBaseBrush
{
public:
	void Spawn( void );
	void Precache( void ){ CBaseBrush::Precache(); }
	void PostActivate ( void );
	void KeyValue( KeyValueData *pkvd );
	void Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value );
	virtual STATE GetState( void ) { return pev->body ? STATE_ON:STATE_OFF; };
	virtual int ObjectCaps( void );
	BOOL OnControls( entvars_t *pevTest );
	CBaseEntity *pCamera;
};
LINK_ENTITY_TO_CLASS( func_monitor, CFuncMonitor );

void CFuncMonitor::KeyValue( KeyValueData *pkvd )
{
	if (FStrEq(pkvd->szKeyName, "camera"))
	{
		pev->netname = ALLOC_STRING( pkvd->szValue );
		pkvd->fHandled = TRUE;
	}
	if (FStrEq(pkvd->szKeyName, "type"))
	{
		pev->impulse = atoi( pkvd->szValue );
		pkvd->fHandled = TRUE;
	}
	if (FStrEq(pkvd->szKeyName, "mode"))
	{
		pev->skin = atoi( pkvd->szValue );
		pkvd->fHandled = TRUE;
	}
	else	CBaseBrush::KeyValue( pkvd );
}

void CFuncMonitor :: Spawn()
{
	m_Material = Computer;
	CBaseBrush::Spawn();

	if(pev->spawnflags & SF_NOTSOLID)//make illusionary wall 
	{
		pev->solid = SOLID_NOT;
		pev->movetype = MOVETYPE_NONE;
	}
	else
	{
		pev->solid = SOLID_BSP;
          	pev->movetype = MOVETYPE_PUSH;
	}
	UTIL_SetModel(ENT(pev), pev->model );

	//enable monitor
	if(pev->spawnflags & SF_START_ON)pev->body = 1;
}

int CFuncMonitor :: ObjectCaps( void ) 
{
	if (pev->impulse)
		return (CBaseEntity::ObjectCaps() & ~FCAP_ACROSS_TRANSITION) | FCAP_IMPULSE_USE;
	else	return (CBaseEntity::ObjectCaps() & ~FCAP_ACROSS_TRANSITION);
}

void CFuncMonitor::PostActivate( void )
{
	//send monitor index
	if(!pCamera)
	{
		MESSAGE_BEGIN( MSG_ALL, gmsgAddScreen );
			WRITE_BYTE( entindex() );
		MESSAGE_END();
	}
	pCamera = UTIL_FindEntityByTargetname(NULL, STRING(pev->netname));
          if(pCamera)pev->scale = pCamera->entindex();
	else if(pev->body) pev->frame = 1;//noise if not found camera
}

void CFuncMonitor :: Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value )
{
	m_hActivator = pActivator;
	
	if(IsLockedByMaster( useType )) return;

	if (useType == USE_TOGGLE)
	{
		if(pev->body) useType = USE_OFF;
		else useType = USE_ON;
	}
	if (useType == USE_ON)pev->body = 1;
	else if (useType == USE_OFF)
	{
		pev->body = 0;
		pev->frame = 0;
	}
	else if (useType == USE_SET)
	{
		if(pActivator->IsPlayer())
		{
			if(pev->body)
			{
				UTIL_SetView( pev->netname, CAMERA_ON );
				m_pController = (CBasePlayer*)pActivator;
				m_pController->m_pMonitor = this;

				if (m_pParent)
					m_vecPlayerPos = m_pController->pev->origin - m_pParent->pev->origin;
				else	m_vecPlayerPos = m_pController->pev->origin;
			}
		}
	}
	else if (useType == USE_RESET)
	{
		if(pActivator->IsPlayer())UTIL_SetView(pActivator, iStringNull, 0 );
	}
	else if (useType == USE_SHOWINFO)
	{
		ALERT(at_console, "======/Xash Debug System/======\n");
		ALERT(at_console, "classname: %s\n", STRING(pev->classname));
		ALERT(at_console, "State: %s, Mode: %s\n", GetStringForState( GetState()), pev->skin ? "Color" : "B&W");
		ALERT(at_console, "Type: %s. Camera Name: %s\n", pev->impulse ? "Duke Nukem Style" : "Half-Life 2 Style", STRING( pev->netname) );
	}
}

BOOL CFuncMonitor :: OnControls( entvars_t *pevTest )
{
	if(m_pParent && ((m_vecPlayerPos + m_pParent->pev->origin) - pevTest->origin).Length() <= 30)
		return TRUE;
	else if((m_vecPlayerPos - pevTest->origin).Length() <= 30 )
		return TRUE;
	return FALSE;
}
