//=======================================================================
//			Copyright (C) Shambler Team 2004
//		         util_.cpp - utility entities: util_fade, 
//			       util_shake, util_fov etc			    
//=======================================================================

#include "extdll.h"
#include "utils.h"
#include "cbase.h"
#include "saverestore.h"
#include "player.h"
#include "defaults.h"
#include "shake.h"

//=======================================================================
// util_setcamera - set view at entity for player
//=======================================================================
class CSetCamera : public CBaseLogic
{
	void Spawn (void );
	void UpdatePlayerView( void );
	void Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value );
	void KeyValue( KeyValueData* );
	int ObjectCaps( void ) { return CBaseLogic::ObjectCaps() & ~FCAP_ACROSS_TRANSITION; }
	char* GetStringForMode( void );
	
};
LINK_ENTITY_TO_CLASS( util_setcamera,  CSetCamera );

void CSetCamera :: KeyValue( KeyValueData* pkvd )
{
	if (FStrEq(pkvd->szKeyName, "lookat"))
	{
		pev->netname = ALLOC_STRING(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else if (FStrEq(pkvd->szKeyName, "hudmode"))
	{
		pev->skin = atoi(pkvd->szValue);
		pkvd->fHandled = TRUE;
	}
	else	CBaseEntity::KeyValue( pkvd );
}

void CSetCamera :: Spawn (void )
{
	pev->solid	         	= SOLID_NOT;
	pev->movetype		= MOVETYPE_NONE;

	switch (pev->skin)
	{
	case 1:	pev->button = 35;	break;//draw normal hud
	case 2:	pev->button = 1;	break;//warhead hud
	case 3:	pev->button = 2;	break;//warhead underwater hud
	case 4:	pev->button = 3;	break;//warhead noise screen
	default:	pev->button = 0;	break;//no hud
	}
	m_iState = STATE_OFF;
}

void CSetCamera :: UpdatePlayerView( void )
{
	int flags;

	((CBasePlayer *)((CBaseEntity *)m_hActivator))->EnableControl(GetState() ? FALSE : TRUE );
	((CBasePlayer *)((CBaseEntity *)m_hActivator))->m_iWarHUD = pev->button;
	if(GetState() == STATE_OFF) flags = 0;
	else flags = (pev->button == 35 ? DRAW_HUD | CAMERA_ON : CAMERA_ON); 
	UTIL_SetView( (CBaseEntity *)m_hActivator, pev->netname, flags );

}

void CSetCamera :: Use( CBaseEntity *pActivator, CBaseEntity *pCaller, USE_TYPE useType, float value )
{
	if(pActivator && pActivator->IsPlayer())//only at player
	{
		m_hActivator = pActivator;//save activator
		
		if (useType == USE_TOGGLE)
		{
			if(m_iState == STATE_OFF) useType = USE_ON;
			else useType = USE_OFF;
		}
		if (useType == USE_ON )
		{
			m_iState = STATE_ON;
			UpdatePlayerView();
		}
		else if (useType == USE_OFF)
		{
			m_iState = STATE_OFF;
			UpdatePlayerView();
		}

	}
	else if (useType == USE_SET)
	{
		//Find func_monitor
		CBaseEntity *pMonitor = UTIL_FindEntityByTargetname(NULL, STRING(pev->target));
		if(pMonitor)
		{
			pMonitor->pev->netname = pev->netname;
			pMonitor->PostActivate();
		}
	}
	else if (useType == USE_RESET)//this is' little easter egg - toggle screen view from player
	{
		//Find func_monitor
		CBaseEntity *pMonitor = UTIL_FindEntityByTargetname(NULL, STRING(pev->target));
		if(pMonitor)
		{
			pMonitor->pev->netname = MAKE_STRING( "player" );
			pMonitor->PostActivate();
		}
	}
	else if (useType == USE_SHOWINFO)
	{
		ALERT(at_console, "======/Xash Debug System/======\n");
		ALERT(at_console, "classname: %s\n", STRING(pev->classname));
		ALERT(at_console, "State: %s, Look at %s\n", GetStringForState( GetState()), STRING(pev->target));
		ALERT(at_console, "HUD Mode: %s\n", GetStringForMode);
	}
}

char* CSetCamera :: GetStringForMode( void )
{
	switch(pev->button)
	{
		case 0: return "Disable HUD";
		case 1: return "Draw Redeemer HUD";
		case 2: return "Draw Redeemer Underwater HUD";
		case 3: return "Draw Redeemer Noise Screen";
		case 35: return "Draw HUD";
	default:	return "";
	}
}

